@@ -1,13 +1,45 @@
-/* assemble.c   code generation for the Netwide Assembler
+/* ----------------------------------------------------------------------- *
+ *   
+ *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   See the file AUTHORS included with the NASM distribution for
+ *   the specific copyright holders.
  *
- * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
- * Julian Hall. All rights reserved. The software is
- * redistributable under the license given in the file "LICENSE"
- * distributed in the NASM archive.
+ *   Redistribution and use in source and binary forms, with or without
+ *   modification, are permitted provided that the following
+ *   conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above
+ *     copyright notice, this list of conditions and the following
+ *     disclaimer in the documentation and/or other materials provided
+ *     with the distribution.
+ *     
+ *     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+ *     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ *     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ *     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ *     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+ *     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ *     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ *     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ *     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ *     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ *     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * ----------------------------------------------------------------------- */
+
+/*
+ * assemble.c   code generation for the Netwide Assembler
  *
  * the actual codes (C syntax, i.e. octal):
  * \0            - terminates the code. (Unless it's a literal of course.)
- * \1, \2, \3    - that many literal bytes follow in the code stream
+ * \1..\4	 - that many literal bytes follow in the code stream
+ * \5            - add 4 to the primary operand number (b, low octdigit)
+ * \6            - add 4 to the secondary operand number (a, middle octdigit)
+ * \7            - add 4 to both the primary and the secondary operand number
  * \10..\13      - a literal byte follows in the code stream, to be added
  *                 to the register value of operand 0..3
  * \14..\17      - a signed byte immediate operand, from operand 0..3
@@ -53,25 +85,26 @@
  *                 is not equal to the truncated and sign-extended 32-bit
  *                 operand; used for 32-bit immediates in 64-bit mode.
  * \254..\257    - a signed 32-bit operand to be extended to 64 bits.
- * \260..\263    - this instruction uses VEX rather than REX, with the
+ * \260..\263    - this instruction uses VEX/XOP rather than REX, with the
  *		   V field taken from operand 0..3.
- * \270		 - this instruction uses VEX rather than REX, with the
+ * \270		 - this instruction uses VEX/XOP rather than REX, with the
  *		   V field set to 1111b.
  *
- * VEX prefixes are followed by the sequence:
- * \mm\wlp         where mm is the M field; and wlp is:
+ * VEX/XOP prefixes are followed by the sequence:
+ * \tmm\wlp        where mm is the M field; and wlp is:
  *                 00 0ww lpp
  *                 [w0] ww = 0 for W = 0
  *                 [w1] ww = 1 for W = 1
  *                 [wx] ww = 2 for W don't care (always assembled as 0)
  *                 [ww] ww = 3 for W used as REX.W
  *
+ * t = 0 for VEX (C4/C5), t = 1 for XOP (8F).
  *
  * \274..\277    - a signed byte immediate operand, from operand 0..3,
  *                 which is to be extended to the operand size.
  * \310          - indicates fixed 16-bit address size, i.e. optional 0x67.
  * \311          - indicates fixed 32-bit address size, i.e. optional 0x67.
- * \312          - (disassembler only) marker on LOOP, LOOPxx instructions.
+ * \312          - (disassembler only) invalid with non-default address size.
  * \313          - indicates fixed 64-bit address size, 0x67 invalid.
  * \314          - (disassembler only) invalid with REX.B
  * \315          - (disassembler only) invalid with REX.X
@@ -84,19 +117,21 @@
  *                 generates no code in the assembler)
  * \323          - indicates fixed 64-bit operand size, REX on extensions only.
  * \324          - indicates 64-bit operand size requiring REX prefix.
+ * \325		 - instruction which always uses spl/bpl/sil/dil
  * \330          - a literal byte follows in the code stream, to be added
  *                 to the condition code value of the instruction.
  * \331          - instruction not valid with REP prefix.  Hint for
  *                 disassembler only; for SSE instructions.
  * \332          - REP prefix (0xF2 byte) used as opcode extension.
  * \333          - REP prefix (0xF3 byte) used as opcode extension.
- * \334          - LOCK prefix used instead of REX.R
+ * \334          - LOCK prefix used as REX.R (used in non-64-bit mode)
  * \335          - disassemble a rep (0xF3 byte) prefix as repe not rep.
  * \336          - force a REP(E) prefix (0xF2) even if not specified.
  * \337		 - force a REPNE prefix (0xF3) even if not specified.
  *                 \336-\337 are still listed as prefixes in the disassembler.
  * \340          - reserve <operand 0> bytes of uninitialized storage.
  *                 Operand 0 had better be a segmentless constant.
+ * \341		 - this instruction needs a WAIT "prefix"
  * \344,\345     - the PUSH/POP (respectively) codes for CS, DS, ES, SS
  *                 (POP is never used for CS) depending on operand 0
  * \346,\347     - the second byte of PUSH/POP codes for FS, GS, depending
@@ -127,9 +162,6 @@
 #include "insns.h"
 #include "tables.h"
 
-/* Initialized to zero by the C standard */
-static const uint8_t const_zero_buf[256];
-
 typedef struct {
     int sib_present;                 /* is a SIB byte necessary? */
     int bytes;                       /* # of bytes of offset needed */
@@ -261,6 +293,11 @@ static bool jmp_match(int32_t segment, int64_t offset, int bits,
 	return false;
 
     isize = calcsize(segment, offset, bits, ins, code);
+
+    if (ins->oprs[0].opflags & OPFLAG_UNKNOWN)
+        /* Be optimistic in pass 1 */
+        return true;
+
     if (ins->oprs[0].segment != segment)
         return false;
 
@@ -350,7 +387,7 @@ int64_t assemble(int32_t segment, int64_t offset, int bits, uint32_t cp,
 
                     if (align) {
                         align = wsize - align;
-                        out(offset, segment, const_zero_buf,
+                        out(offset, segment, zero_buffer,
                             OUT_RAWDATA, align, NO_SEG, NO_SEG);
                     }
                     offset += e->stringlen + align;
@@ -407,9 +444,8 @@ int64_t assemble(int32_t segment, int64_t offset, int bits, uint32_t cp,
                 fseek(fp, base, SEEK_SET);
                 l = len;
                 while (l > 0) {
-                    int32_t m =
-                        fread(buf, 1, (l > (int32_t) sizeof(buf) ? (int32_t) sizeof(buf) : l),
-                              fp);
+                    int32_t m;
+		    m = fread(buf, 1, l > sizeof(buf) ? sizeof(buf) : l, fp);
                     if (!m) {
                         /*
                          * This shouldn't happen unless the file
@@ -446,7 +482,7 @@ int64_t assemble(int32_t segment, int64_t offset, int bits, uint32_t cp,
     /* Check to see if we need an address-size prefix */
     add_asp(instruction, bits);
 
-    size_prob = false;
+    size_prob = 0;
 
     for (temp = nasm_instructions[instruction->opcode]; temp->opcode != -1; temp++){
         int m = matches(temp, instruction, bits);
@@ -464,6 +500,9 @@ int64_t assemble(int32_t segment, int64_t offset, int bits, uint32_t cp,
                     for (j = 0; j < MAXPREFIX; j++) {
                         uint8_t c = 0;
                         switch (instruction->prefixes[j]) {
+			case P_WAIT:
+			    c = 0x9B;
+			    break;
                         case P_LOCK:
                             c = 0xF0;
                             break;
@@ -595,7 +634,8 @@ int64_t assemble(int32_t segment, int64_t offset, int bits, uint32_t cp,
             error(ERR_NONFATAL, "no instruction for this cpu level");
 	    break;
 	case 4:
-            error(ERR_NONFATAL, "instruction not supported in 64-bit mode");
+            error(ERR_NONFATAL, "instruction not supported in %d-bit mode",
+		  bits);
 	    break;
 	default:
             error(ERR_NONFATAL,
@@ -748,7 +788,7 @@ int64_t insn_size(int32_t segment, int64_t offset, int bits, uint32_t cp,
 static bool possible_sbyte(operand *o)
 {
     return o->wrt == NO_SEG && o->segment == NO_SEG &&
-	!(o->opflags & OPFLAG_FORWARD) &&
+	!(o->opflags & OPFLAG_UNKNOWN) &&
 	optimizing >= 0 && !(o->type & STRICT);
 }
 
@@ -784,7 +824,9 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
     int64_t length = 0;
     uint8_t c;
     int rex_mask = ~0;
+    int op1, op2;
     struct operand *opx;
+    uint8_t opex = 0;
 
     ins->rex = 0;               /* Ensure REX is reset */
 
@@ -796,14 +838,25 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
 
     while (*codes) {
 	c = *codes++;
-	opx = &ins->oprs[c & 3];
+	op1 = (c & 3) + ((opex & 1) << 2);
+	op2 = ((c >> 3) & 3) + ((opex & 2) << 1);
+	opx = &ins->oprs[op1];
+	opex = 0;		/* For the next iteration */
+
         switch (c) {
         case 01:
         case 02:
         case 03:
+	case 04:
             codes += c, length += c;
             break;
 
+	case 05:
+	case 06:
+	case 07:
+	    opex = c;
+	    break;
+
 	case4(010):
 	    ins->rex |=
 		op_rexflags(opx, REX_B|REX_H|REX_P|REX_W);
@@ -913,14 +966,14 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
 	case4(0260):
 	    ins->rex |= REX_V;
 	    ins->drexdst = regval(opx);
-	    ins->vex_m = *codes++;
+	    ins->vex_cm = *codes++;
 	    ins->vex_wlp = *codes++;
 	    break;
 
 	case 0270:
 	    ins->rex |= REX_V;
 	    ins->drexdst = 0;
-	    ins->vex_m = *codes++;
+	    ins->vex_cm = *codes++;
 	    ins->vex_wlp = *codes++;
 	    break;
 
@@ -972,6 +1025,10 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
 	    ins->rex |= REX_W;
             break;
 
+	case 0325:
+	    ins->rex |= REX_NH;
+	    break;
+
         case 0330:
             codes++, length++;
             break;
@@ -1009,6 +1066,11 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
                 length += ins->oprs[0].offset;
             break;
 
+	case 0341:
+	    if (!ins->prefixes[PPS_WAIT])
+		ins->prefixes[PPS_WAIT] = P_WAIT;
+	    break;
+
 	case4(0344):
             length++;
             break;
@@ -1056,20 +1118,20 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
                 ea ea_data;
                 int rfield;
 		int32_t rflags;
+		struct operand *opy = &ins->oprs[op2];
+
                 ea_data.rex = 0;           /* Ensure ea.REX is initially 0 */
 
 		if (c <= 0177) {
-		    /* pick rfield from operand b */
-		    rflags = regflag(&ins->oprs[c & 7]);
-		    rfield = nasm_regvals[ins->oprs[c & 7].basereg];
+		    /* pick rfield from operand b (opx) */
+		    rflags = regflag(opx);
+		    rfield = nasm_regvals[opx->basereg];
 		} else {
 		    rflags = 0;
 		    rfield = c & 7;
 		}
-
-                if (!process_ea
-                    (&ins->oprs[(c >> 3) & 7], &ea_data, bits,
-		     ins->addr_size, rfield, rflags)) {
+                if (!process_ea(opy, &ea_data, bits,
+				ins->addr_size, rfield, rflags)) {
                     errfunc(ERR_NONFATAL, "invalid effective address");
                     return -1;
                 } else {
@@ -1081,13 +1143,21 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
 
 	default:
 	    errfunc(ERR_PANIC, "internal instruction table corrupt"
-		    ": instruction code 0x%02X given", c);
+		    ": instruction code \\%o (0x%02X) given", c, c);
 	    break;
 	}
     }
 
     ins->rex &= rex_mask;
 
+    if (ins->rex & REX_NH) {
+	if (ins->rex & REX_H) {
+	    errfunc(ERR_NONFATAL, "instruction cannot use high registers");
+	    return -1;
+	}
+	ins->rex &= ~REX_P;	/* Don't force REX prefix due to high reg */
+    }
+
     if (ins->rex & REX_V) {
 	int bad32 = REX_R|REX_W|REX_X|REX_B;
 
@@ -1113,7 +1183,7 @@ static int64_t calcsize(int32_t segment, int64_t offset, int bits,
 	    errfunc(ERR_NONFATAL, "invalid operands in non-64-bit mode");
 	    return -1;
 	}
-	if (ins->vex_m != 1 || (ins->rex & (REX_W|REX_R|REX_B)))
+	if (ins->vex_cm != 1 || (ins->rex & (REX_W|REX_R|REX_B)))
 	    length += 3;
 	else
 	    length += 2;
@@ -1170,25 +1240,38 @@ static void gencode(int32_t segment, int64_t offset, int bits,
     uint8_t bytes[4];
     int64_t size;
     int64_t data;
+    int op1, op2;
     struct operand *opx;
     const uint8_t *codes = temp->code;
+    uint8_t opex = 0;
 
     while (*codes) {
 	c = *codes++;
-	opx = &ins->oprs[c & 3];
+	op1 = (c & 3) + ((opex & 1) << 2);
+	op2 = ((c >> 3) & 3) + ((opex & 2) << 1);
+	opx = &ins->oprs[op1];
+	opex = 0;		/* For the next iteration */
+
         switch (c) {
         case 01:
         case 02:
         case 03:
+	case 04:
 	    EMIT_REX();
             out(offset, segment, codes, OUT_RAWDATA, c, NO_SEG, NO_SEG);
             codes += c;
             offset += c;
             break;
 
+	case 05:
+	case 06:
+	case 07:
+	    opex = c;
+	    break;
+
 	case4(010):
 	    EMIT_REX();
-            bytes[0] = *codes++ + ((regval(opx)) & 7);
+            bytes[0] = *codes++ + (regval(opx) & 7);
             out(offset, segment, bytes, OUT_RAWDATA, 1, NO_SEG, NO_SEG);
             offset += 1;
             break;
@@ -1199,7 +1282,7 @@ static void gencode(int32_t segment, int64_t offset, int bits,
 	       warn on explicit BYTE directives.  Also warn, obviously,
 	       if the optimizer isn't enabled. */
             if (((opx->type & BITS8) ||
-		 !(opx->type & temp->opd[c & 3] & BYTENESS)) &&
+		 !(opx->type & temp->opd[op1] & BYTENESS)) &&
 		(opx->offset < -128 || opx->offset > 127)) {
                 errfunc(ERR_WARNING | ERR_PASS2 | ERR_WARN_NOV,
 			"signed byte value exceeds bounds");
@@ -1495,9 +1578,9 @@ static void gencode(int32_t segment, int64_t offset, int bits,
 	case4(0260):
 	case 0270:
 	    codes += 2;
-	    if (ins->vex_m != 1 || (ins->rex & (REX_W|REX_X|REX_B))) {
-		bytes[0] = 0xc4;
-		bytes[1] = ins->vex_m | ((~ins->rex & 7) << 5);
+	    if (ins->vex_cm != 1 || (ins->rex & (REX_W|REX_X|REX_B))) {
+		bytes[0] = (ins->vex_cm >> 6) ? 0x8f : 0xc4;
+		bytes[1] = (ins->vex_cm & 31) | ((~ins->rex & 7) << 5);
 		bytes[2] = ((ins->rex & REX_W) << (7-3)) |
 		    ((~ins->drexdst & 15)<< 3) | (ins->vex_wlp & 07);
 		out(offset, segment, &bytes, OUT_RAWDATA, 3, NO_SEG, NO_SEG);
@@ -1603,6 +1686,9 @@ static void gencode(int32_t segment, int64_t offset, int bits,
             ins->rex |= REX_W;
             break;
 
+	case 0325:
+	    break;
+
         case 0330:
             *bytes = *codes++ ^ condval[ins->condition];
             out(offset, segment, bytes, OUT_RAWDATA, 1, NO_SEG, NO_SEG);
@@ -1647,6 +1733,9 @@ static void gencode(int32_t segment, int64_t offset, int bits,
             }
             break;
 
+	case 0341:
+	    break;
+
         case 0344:
         case 0345:
 	    bytes[0] = c & 1;
@@ -1746,20 +1835,20 @@ static void gencode(int32_t segment, int64_t offset, int bits,
                 uint8_t *p;
                 int32_t s;
 		enum out_type type;
+		struct operand *opy = &ins->oprs[op2];
 
                 if (c <= 0177) {
-		    /* pick rfield from operand b */
-		    rflags = regflag(&ins->oprs[c & 7]);
-                    rfield = nasm_regvals[ins->oprs[c & 7].basereg];
+		    /* pick rfield from operand b (opx) */
+		    rflags = regflag(opx);
+                    rfield = nasm_regvals[opx->basereg];
 		} else {
 		    /* rfield is constant */
 		    rflags = 0;
                     rfield = c & 7;
 		}
 
-                if (!process_ea
-                    (&ins->oprs[(c >> 3) & 7], &ea_data, bits,
-		     ins->addr_size, rfield, rflags)) {
+                if (!process_ea(opy, &ea_data, bits, ins->addr_size,
+				rfield, rflags)) {
                     errfunc(ERR_NONFATAL, "invalid effective address");
                 }
 
@@ -1771,10 +1860,9 @@ static void gencode(int32_t segment, int64_t offset, int bits,
 
 		/* DREX suffixes come between the SIB and the displacement */
 		if (ins->rex & REX_D) {
-		    *p++ =
-			(ins->drexdst << 4) |
-			(ins->rex & REX_OC ? 0x08 : 0) |
-			(ins->rex & (REX_R|REX_X|REX_B));
+		    *p++ = (ins->drexdst << 4) |
+			   (ins->rex & REX_OC ? 0x08 : 0) |
+			   (ins->rex & (REX_R|REX_X|REX_B));
 		    ins->rex = 0;
 		}
 
@@ -1792,44 +1880,33 @@ static void gencode(int32_t segment, int64_t offset, int bits,
                 case 0:
                     break;
                 case 1:
-                    if (ins->oprs[(c >> 3) & 7].segment != NO_SEG) {
-                        data = ins->oprs[(c >> 3) & 7].offset;
-                        out(offset, segment, &data, OUT_ADDRESS, 1,
-                            ins->oprs[(c >> 3) & 7].segment,
-                            ins->oprs[(c >> 3) & 7].wrt);
-                    } else {
-                        *bytes = ins->oprs[(c >> 3) & 7].offset;
-                        out(offset, segment, bytes, OUT_RAWDATA, 1,
-                            NO_SEG, NO_SEG);
-                    }
-                    s++;
-                    break;
-                case 8:
                 case 2:
                 case 4:
-                    data = ins->oprs[(c >> 3) & 7].offset;
-		    warn_overflow(ea_data.bytes, &ins->oprs[(c >> 3) & 7]);
+                case 8:
+                    data = opy->offset;
+		    warn_overflow(ea_data.bytes, opy);
                     s += ea_data.bytes;
 		    if (ea_data.rip) {
-			if (ins->oprs[(c >> 3) & 7].segment == segment) {
+			if (opy->segment == segment) {
 			    data -= insn_end;
-			    out(offset, segment, &data,
-				OUT_ADDRESS, ea_data.bytes,
-				NO_SEG, NO_SEG);
+			    out(offset, segment, &data, OUT_ADDRESS,
+				ea_data.bytes, NO_SEG, NO_SEG);
 			} else {
-			    out(offset, segment, &data,
-				OUT_REL4ADR, insn_end - offset,
-				ins->oprs[(c >> 3) & 7].segment,
-				ins->oprs[(c >> 3) & 7].wrt);
+			    out(offset, segment, &data,	OUT_REL4ADR,
+				insn_end - offset, opy->segment, opy->wrt);
 			}
 		    } else {
 			type = OUT_ADDRESS;
-			out(offset, segment, &data,
-			    OUT_ADDRESS, ea_data.bytes,
-			    ins->oprs[(c >> 3) & 7].segment,
-			    ins->oprs[(c >> 3) & 7].wrt);
+			out(offset, segment, &data, OUT_ADDRESS,
+			    ea_data.bytes, opy->segment, opy->wrt);
 		    }
                     break;
+                default:
+                    /* Impossible! */
+                    errfunc(ERR_PANIC,
+                            "Invalid amount of bytes (%d) for offset?!",
+                            ea_data.bytes);
+                    break;
                 }
                 offset += s;
 	    }
@@ -1837,7 +1914,7 @@ static void gencode(int32_t segment, int64_t offset, int bits,
 
 	default:
 	    errfunc(ERR_PANIC, "internal instruction table corrupt"
-		    ": instruction code 0x%02X given", c);
+		    ": instruction code \\%o (0x%02X) given", c, c);
 	    break;
         }
     }
@@ -2054,9 +2131,9 @@ static int matches(const struct itemplate *itemp, insn * instruction, int bits)
         return 3;
 
     /*
-     * Check if instruction is available in long mode
+     * Verify the appropriate long mode flag.
      */
-    if ((itemp->flags & IF_NOLONG) && (bits == 64))
+    if ((itemp->flags & (bits == 64 ? IF_NOLONG : IF_LONG)))
         return 4;
 
     /*
@@ -2071,7 +2148,7 @@ static int matches(const struct itemplate *itemp, insn * instruction, int bits)
 static ea *process_ea(operand * input, ea * output, int bits,
 		      int addrbits, int rfield, int32_t rflags)
 {
-    bool forw_ref = !!(input->opflags & OPFLAG_FORWARD);
+    bool forw_ref = !!(input->opflags & OPFLAG_UNKNOWN);
 
     output->rip = false;
 
