@@ -1,11 +1,38 @@
-/* float.c     floating-point constant support for the Netwide Assembler
+/* ----------------------------------------------------------------------- *
+ *   
+ *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   See the file AUTHORS included with the NASM distribution for
+ *   the specific copyright holders.
  *
- * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
- * Julian Hall. All rights reserved. The software is
- * redistributable under the license given in the file "LICENSE"
- * distributed in the NASM archive.
+ *   Redistribution and use in source and binary forms, with or without
+ *   modification, are permitted provided that the following
+ *   conditions are met:
  *
- * initial version 13/ix/96 by Simon Tatham
+ *   * Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above
+ *     copyright notice, this list of conditions and the following
+ *     disclaimer in the documentation and/or other materials provided
+ *     with the distribution.
+ *     
+ *     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+ *     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ *     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ *     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ *     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+ *     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ *     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ *     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ *     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ *     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ *     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * ----------------------------------------------------------------------- */
+
+/*
+ * float.c     floating-point constant support for the Netwide Assembler
  */
 
 #include "compiler.h"
@@ -45,15 +72,6 @@ typedef uint64_t fp_2limb;
 #define LIMB_ALL_BYTES	((fp_limb)0x01010101)
 #define LIMB_BYTE(x)	((x)*LIMB_ALL_BYTES)
 
-#if X86_MEMORY
-#define put(a,b) (*(uint32_t *)(a) = (b))
-#else
-#define put(a,b) (((a)[0] = (b)),		\
-		  ((a)[1] = (b) >> 8),		\
-		  ((a)[2] = (b) >> 16),		\
-		  ((a)[3] = (b) >> 24))
-#endif
-
 /* 112 bits + 64 bits for accuracy + 16 bits for rounding */
 #define MANT_LIMBS 6
 
@@ -501,7 +519,7 @@ static bool ieee_flconvert_bin(const char *string, int bits,
     int32_t twopwr;
     bool seendot, seendigit;
     unsigned char c;
-    int radix = 1 << bits;
+    const int radix = 1 << bits;
     fp_limb v;
 
     twopwr = 0;
@@ -565,10 +583,10 @@ static bool ieee_flconvert_bin(const char *string, int bits,
     }
 
     if (!seendigit) {
-        memset(mant, 0, sizeof mult); /* Zero */
+        memset(mant, 0, MANT_LIMBS*sizeof(fp_limb)); /* Zero */
         *exponent = 0;
     } else {
-        memcpy(mant, mult, sizeof mult);
+        memcpy(mant, mult, MANT_LIMBS*sizeof(fp_limb));
         *exponent = twopwr;
     }
 
@@ -647,21 +665,83 @@ enum floats {
     FL_SNAN
 };
 
-static int to_float(const char *str, int s, uint8_t * result,
+static int to_packed_bcd(const char *str, const char *p,
+			 int s, uint8_t *result,
+			 const struct ieee_format *fmt)
+{
+    int n = 0;
+    char c;
+    int tv = -1;
+
+    if (fmt != &ieee_80) {
+	error(ERR_NONFATAL|ERR_PASS1,
+	      "packed BCD requires an 80-bit format");
+	return 0;
+    }
+
+    while (p >= str) {
+	c = *p--;
+	if (c >= '0' && c <= '9') {
+	    if (tv < 0) {
+		if (n == 9) {
+		    error(ERR_WARNING|ERR_PASS1,
+			  "packed BCD truncated to 18 digits");
+		}
+		tv = c-'0';
+	    } else {
+		if (n < 9)
+		    *result++ = tv + ((c-'0') << 4);
+		n++;
+		tv = -1;
+	    }
+	} else if (c == '_') {
+	    /* do nothing */
+	} else {
+	    error(ERR_NONFATAL|ERR_PASS1,
+		  "invalid character `%c' in packed BCD constant", c);
+	    return 0;
+	}
+    }
+    if (tv >= 0) {
+	if (n < 9)
+	    *result++ = tv;
+	n++;
+    }
+    while (n < 9) {
+	*result++ = 0;
+	n++;
+    }
+    *result = (s < 0) ? 0x80 : 0;
+
+    return 1;			/* success */
+}
+
+static int to_float(const char *str, int s, uint8_t *result,
                     const struct ieee_format *fmt)
 {
-    fp_limb mant[MANT_LIMBS], *mp, m;
+    fp_limb mant[MANT_LIMBS];
     int32_t exponent = 0;
-    int32_t expmax = 1 << (fmt->exponent - 1);
+    const int32_t expmax = 1 << (fmt->exponent - 1);
     fp_limb one_mask = LIMB_TOP_BIT >>
 	((fmt->exponent+fmt->explicit) % LIMB_BITS);
-    int one_pos = (fmt->exponent+fmt->explicit)/LIMB_BITS;
+    const int one_pos = (fmt->exponent+fmt->explicit)/LIMB_BITS;
     int i;
     int shift;
     enum floats type;
     bool ok;
-    bool minus = s < 0;
-    int bits = fmt->bytes * 8;
+    const bool minus = s < 0;
+    const int bits = fmt->bytes * 8;
+    const char *strend;
+
+    if (!str[0]) {
+	error(ERR_PANIC,
+	      "internal errror: empty string passed to float_const");
+	return 0;
+    }
+
+    strend = strchr(str, '\0');
+    if (strend[-1] == 'P' || strend[-1] == 'p')
+	return to_packed_bcd(str, strend-2, s, result, fmt);
 
     if (str[0] == '_') {
 	/* Special tokens */
@@ -706,6 +786,8 @@ static int to_float(const char *str, int s, uint8_t * result,
 	    case 't': case 'T':
 		ok = ieee_flconvert(str+2, mant, &exponent);
 		break;
+	    case 'p': case 'P':
+		return to_packed_bcd(str+2, strend-1, s, result, fmt);
 	    default:
 		/* Leading zero was just a zero? */
 		ok = ieee_flconvert(str, mant, &exponent);
@@ -813,21 +895,13 @@ static int to_float(const char *str, int s, uint8_t * result,
 
     mant[0] |= minus ? LIMB_TOP_BIT : 0;
 
-    m = mant[fmt->bytes/LIMB_BYTES];
-    for (i = LIMB_BYTES-(fmt->bytes % LIMB_BYTES); i < LIMB_BYTES; i++)
-	*result++ = m >> (i*8);
-
-    for (mp = &mant[fmt->bytes/LIMB_BYTES], i = 0;
-	 i < fmt->bytes; i += LIMB_BYTES) {
-        m = *--mp;
-        put(result, m);
-        result += LIMB_BYTES;
-    }
+    for (i = fmt->bytes - 1; i >= 0; i--)
+	*result++ = mant[i/LIMB_BYTES] >> (((LIMB_BYTES-1)-(i%LIMB_BYTES))*8);
 
     return 1;                   /* success */
 }
 
-int float_const(const char *number, int sign, uint8_t * result,
+int float_const(const char *number, int sign, uint8_t *result,
                 int bytes, efunc err)
 {
     error = err;
