@@ -1,11 +1,41 @@
-/* outelf64.c	output routines for the Netwide Assembler to produce
- *		ELF64 (x86_64 of course) object file format
+/* ----------------------------------------------------------------------- *
+ *   
+ *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   See the file AUTHORS included with the NASM distribution for
+ *   the specific copyright holders.
  *
- * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
- * Julian Hall. All rights reserved. The software is
- * redistributable under the license given in the file "LICENSE"
- * distributed in the NASM archive.
+ *   Redistribution and use in source and binary forms, with or without
+ *   modification, are permitted provided that the following
+ *   conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above
+ *     copyright notice, this list of conditions and the following
+ *     disclaimer in the documentation and/or other materials provided
+ *     with the distribution.
+ *     
+ *     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+ *     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ *     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ *     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ *     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+ *     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ *     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ *     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ *     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ *     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ *     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * ----------------------------------------------------------------------- */
+
+/*
+ * outelf64.c	output routines for the Netwide Assembler to produce
+ *		ELF64 (x86_64 of course) object file format
  */
+
 #include "compiler.h"
 
 #include <stdio.h>
@@ -19,116 +49,17 @@
 #include "saa.h"
 #include "raa.h"
 #include "stdscan.h"
-#include "outform.h"
-
-/* Definitions in lieu of elf.h */
-#define SHT_NULL 0			/* Inactive section header */
-#define SHT_PROGBITS 1			/* Program defined content */
-#define SHT_RELA	  4		/* Relocation entries with addends */
-#define SHT_NOBITS 8			/* Section requires no space in file */
-#define SHF_WRITE	     (1 << 0)	/* Writable */
-#define SHF_ALLOC	     (1 << 1)	/* Occupies memory during execution */
-#define SHF_EXECINSTR	     (1 << 2)	/* Executable */
-#define SHN_ABS		0xfff1		/* Associated symbol is absolute */
-#define SHN_COMMON	0xfff2		/* Associated symbol is common */
-#define R_X86_64_NONE		0	/* No reloc */
-#define R_X86_64_64		1	/* Direct 64 bit  */
-#define R_X86_64_PC32		2	/* PC relative 32 bit signed */
-#define R_X86_64_GOT32		3	/* 32 bit GOT entry */
-#define R_X86_64_PLT32		4	/* 32 bit PLT address */
-#define R_X86_64_COPY		5	/* Copy symbol at runtime */
-#define R_X86_64_GLOB_DAT	6	/* Create GOT entry */
-#define R_X86_64_JUMP_SLOT	7	/* Create PLT entry */
-#define R_X86_64_RELATIVE	8	/* Adjust by program base */
-#define R_X86_64_GOTPCREL	9	/* 32 bit signed PC relative
-					   offset to GOT */
-#define R_X86_64_32		10	/* Direct 32 bit zero extended */
-#define R_X86_64_32S		11	/* Direct 32 bit sign extended */
-#define R_X86_64_16		12	/* Direct 16 bit zero extended */
-#define R_X86_64_PC16		13	/* 16 bit sign extended pc relative */
-#define R_X86_64_8		14	/* Direct 8 bit sign extended  */
-#define R_X86_64_PC8		15	/* 8 bit sign extended pc relative */
-#define R_X86_64_DTPMOD64	16	/* ID of module containing symbol */
-#define R_X86_64_DTPOFF64	17	/* Offset in module's TLS block */
-#define R_X86_64_TPOFF64	18	/* Offset in initial TLS block */
-#define R_X86_64_TLSGD		19	/* 32 bit signed PC relative offset
-					   to two GOT entries for GD symbol */
-#define R_X86_64_TLSLD		20	/* 32 bit signed PC relative offset
-					   to two GOT entries for LD symbol */
-#define R_X86_64_DTPOFF32	21	/* Offset in TLS block */
-#define R_X86_64_GOTTPOFF	22	/* 32 bit signed PC relative offset
-					   to GOT entry for IE symbol */
-#define R_X86_64_TPOFF32	23	/* Offset in initial TLS block */
-#define R_X86_64_PC64		24 	/* word64 S + A - P */
-#define R_X86_64_GOTOFF64	25 	/* word64 S + A - GOT */
-#define R_X86_64_GOTPC32	26 	/* word32 GOT + A - P */
-#define R_X86_64_GOT64		27 	/* word64 G + A */
-#define R_X86_64_GOTPCREL64	28 	/* word64 G + GOT - P + A */
-#define R_X86_64_GOTPC64	29 	/* word64 GOT - P + A */
-#define R_X86_64_GOTPLT64	30 	/* word64 G + A */
-#define R_X86_64_PLTOFF64	31 	/* word64 L - GOT + A */
-#define R_X86_64_SIZE32		32 	/* word32 Z + A */
-#define R_X86_64_SIZE64		33 	/* word64 Z + A */
-#define R_X86_64_GOTPC32_TLSDESC 34 	/* word32 */
-#define R_X86_64_TLSDESC_CALL	35 	/* none */
-#define R_X86_64_TLSDESC	36 	/* word64×2 */
-#define ET_REL		1		/* Relocatable file */
-#define EM_X86_64	62		/* AMD x86-64 architecture */
-#define STT_NOTYPE	0		/* Symbol type is unspecified */
-#define STT_OBJECT	1		/* Symbol is a data object */
-#define STT_FUNC	2		/* Symbol is a code object */
-#define STT_SECTION	3		/* Symbol associated with a section */
-#define STT_FILE	4		/* Symbol's name is file name */
-#define STT_COMMON	5		/* Symbol is a common data object */
-#define STT_TLS		6		/* Symbol is thread-local data object*/
-#define	STT_NUM		7		/* Number of defined types.  */
-
-/* Definitions in lieu of dwarf.h */
-#define    DW_TAG_compile_unit   0x11
-#define    DW_TAG_subprogram   0x2e
-#define    DW_AT_name   0x03
-#define    DW_AT_stmt_list   0x10
-#define    DW_AT_low_pc   0x11
-#define    DW_AT_high_pc   0x12
-#define    DW_AT_language  0x13
-#define    DW_AT_producer   0x25
-#define    DW_AT_frame_base   0x40
-#define    DW_FORM_addr   0x01
-#define    DW_FORM_data2   0x05
-#define    DW_FORM_data4   0x06
-#define    DW_FORM_string   0x08
-#define    DW_LNS_extended_op  0
-#define    DW_LNS_advance_pc   2
-#define    DW_LNS_advance_line   3
-#define    DW_LNS_set_file   4
-#define    DW_LNE_end_sequence   1
-#define    DW_LNE_set_address   2
-#define    DW_LNE_define_file   3
-#define    DW_LANG_Mips_Assembler  0x8001
-
-#define SOC(ln,aa) ln - line_base + (line_range * aa) + opcode_base
-
-typedef uint32_t Elf64_Word;
-typedef uint64_t Elf64_Xword;
-typedef uint64_t Elf64_Addr;
-typedef uint64_t Elf64_Off;
-typedef struct
-{
-  Elf64_Word	sh_name;	/* Section name (string tbl index) */
-  Elf64_Word	sh_type;	/* Section type */
-  Elf64_Xword	sh_flags;	/* Section flags */
-  Elf64_Addr	sh_addr;	/* Section virtual addr at execution */
-  Elf64_Off	sh_offset;	/* Section file offset */
-  Elf64_Xword	sh_size;	/* Section size in bytes */
-  Elf64_Word	sh_link;	/* Link to another section */
-  Elf64_Word	sh_info;	/* Additional section information */
-  Elf64_Xword	sh_addralign;	/* Section alignment */
-  Elf64_Xword	sh_entsize;	/* Entry size if section holds table */
-} Elf64_Shdr;
+#include "output/outform.h"
+#include "output/outlib.h"
+#include "rbtree.h"
 
+#include "output/elf64.h"
+#include "output/dwarf.h"
+#include "output/outelf.h"
 
 #ifdef OF_ELF64
 
+#define SOC(ln,aa) ln - line_base + (line_range * aa) + opcode_base
 
 struct Reloc {
     struct Reloc *next;
@@ -139,32 +70,30 @@ struct Reloc {
 };
 
 struct Symbol {
+    struct rbtree symv;	        /* symbol value and rbtree of globals */
     int32_t strpos;             /* string table position of name */
     int32_t section;            /* section ID of the symbol */
     int type;			/* symbol type */
     int other;                  /* symbol visibility */
-    int64_t value;              /* address, or COMMON variable align */
     int32_t size;               /* size of symbol */
     int32_t globnum;            /* symbol table offset if global */
-    struct Symbol *next;       	/* list of globals in each section */
     struct Symbol *nextfwd;     /* list of unresolved-size symbols */
     char *name;			/* used temporarily if in above list */
 };
 
-
 struct Section {
     struct SAA *data;
     uint64_t len, size;
     uint32_t nrelocs;
     int32_t index;	       	/* index into sects array */
-    uint32_t type;             	/* SHT_PROGBITS or SHT_NOBITS */
+    int type;             	/* SHT_PROGBITS or SHT_NOBITS */
     uint64_t align;            	/* alignment: power of two */
     uint64_t flags;            	/* section flags */
     char *name;
     struct SAA *rel;
     uint64_t rellen;
     struct Reloc *head, **tail;
-    struct Symbol *gsyms;       /* global symbols in section */
+    struct rbtree *gsyms;       /* global symbols in section */
 };
 
 #define SECT_DELTA 32
@@ -176,7 +105,7 @@ static char *shstrtab;
 static int shstrtablen, shstrtabsize;
 
 static struct SAA *syms;
-static uint32_t nlocals, nglobs;
+static uint32_t nlocals, nglobs, ndebugs;
 
 static int32_t def_seg;
 
@@ -198,24 +127,6 @@ static uint8_t elf_abiver = 0;	/* Current ABI version */
 
 extern struct ofmt of_elf64;
 
-#define SHN_UNDEF 0
-
-#define SYM_GLOBAL 0x10
-
-#define STV_DEFAULT 0
-#define STV_INTERNAL 1
-#define STV_HIDDEN 2
-#define STV_PROTECTED 3
-
-#define GLOBAL_TEMP_BASE 1048576     /* bigger than any reasonable sym id */
-
-#define SEG_ALIGN 16            /* alignment of sections in file */
-#define SEG_ALIGN_1 (SEG_ALIGN-1)
-
-#define TY_DEBUGSYMLIN 0x40     /* internal call to debug_out */
-
-static const char align_str[SEG_ALIGN] = "";    /* ANSI will pad this with 0s */
-
 static struct ELF_SECTDATA {
     void *data;
     int64_t len;
@@ -307,24 +218,24 @@ static struct dfmt df_stabs;
 static struct Symbol *lastsym;
 
 /* common debugging routines */
-void debug64_typevalue(int32_t);
-void debug64_init(struct ofmt *, void *, FILE *, efunc);
-void debug64_deflabel(char *, int32_t, int64_t, int, char *);
-void debug64_directive(const char *, const char *);
+static void debug64_typevalue(int32_t);
+static void debug64_deflabel(char *, int32_t, int64_t, int, char *);
+static void debug64_directive(const char *, const char *);
 
 /* stabs debugging routines */
-void stabs64_linenum(const char *filename, int32_t linenumber, int32_t);
-void stabs64_output(int, void *);
-void stabs64_generate(void);
-void stabs64_cleanup(void);
+static void stabs64_linenum(const char *filename, int32_t linenumber, int32_t);
+static void stabs64_output(int, void *);
+static void stabs64_generate(void);
+static void stabs64_cleanup(void);
 
 /* dwarf debugging routines */
-void dwarf64_linenum(const char *filename, int32_t linenumber, int32_t);
-void dwarf64_output(int, void *);
-void dwarf64_generate(void);
-void dwarf64_cleanup(void);
-void dwarf64_findfile(const char *);
-void dwarf64_findsect(const int);
+static void dwarf64_init(struct ofmt *, void *, FILE *, efunc);
+static void dwarf64_linenum(const char *filename, int32_t linenumber, int32_t);
+static void dwarf64_output(int, void *);
+static void dwarf64_generate(void);
+static void dwarf64_cleanup(void);
+static void dwarf64_findfile(const char *);
+static void dwarf64_findsect(const int);
 
 /*
  * Special section numbers which are used to define ELF special
@@ -334,6 +245,7 @@ void dwarf64_findsect(const int);
 static int32_t elf_gotpc_sect, elf_gotoff_sect;
 static int32_t elf_got_sect, elf_plt_sect;
 static int32_t elf_sym_sect;
+static int32_t elf_gottpoff_sect;
 
 static void elf_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
 {
@@ -345,7 +257,7 @@ static void elf_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
     sects = NULL;
     nsects = sectlen = 0;
     syms = saa_init((int32_t)sizeof(struct Symbol));
-    nlocals = nglobs = 0;
+    nlocals = nglobs = ndebugs = 0;
     bsym = raa_init();
     strs = saa_init(1L);
     saa_wbytes(strs, "\0", 1L);
@@ -372,6 +284,9 @@ static void elf_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
     elf_sym_sect = seg_alloc();
     ldef("..sym", elf_sym_sect + 1, 0L, NULL, false, false, &of_elf64,
          error);
+    elf_gottpoff_sect = seg_alloc();
+    ldef("..gottpoff", elf_gottpoff_sect + 1, 0L, NULL, false, false, &of_elf64,
+         error);
 
     def_seg = seg_alloc();
 
@@ -441,8 +356,7 @@ static int elf_make_section(char *name, int type, int flags, int align)
     s->gsyms = NULL;
 
     if (nsects >= sectlen)
-        sects =
-            nasm_realloc(sects, (sectlen += SECT_DELTA) * sizeof(*sects));
+        sects = nasm_realloc(sects, (sectlen += SECT_DELTA) * sizeof(*sects));
     sects[nsects++] = s;
 
     return nsects - 1;
@@ -451,9 +365,9 @@ static int elf_make_section(char *name, int type, int flags, int align)
 static int32_t elf_section_names(char *name, int pass, int *bits)
 {
     char *p;
-    unsigned flags_and, flags_or;
-    uint64_t type, align;
-    int i;
+    uint32_t flags, flags_and, flags_or;
+    uint64_t align;
+    int type, i;
 
     /*
      * Default is 64 bits.
@@ -505,6 +419,9 @@ static int32_t elf_section_names(char *name, int pass, int *bits)
         } else if (!nasm_stricmp(q, "write")) {
             flags_and |= SHF_WRITE;
             flags_or |= SHF_WRITE;
+        } else if (!nasm_stricmp(q, "tls")) {
+            flags_and |= SHF_TLS;
+            flags_or |= SHF_TLS;
         } else if (!nasm_stricmp(q, "nowrite")) {
             flags_and |= SHF_WRITE;
             flags_or &= ~SHF_WRITE;
@@ -512,12 +429,15 @@ static int32_t elf_section_names(char *name, int pass, int *bits)
             type = SHT_PROGBITS;
         } else if (!nasm_stricmp(q, "nobits")) {
             type = SHT_NOBITS;
-        }
+        } else if (pass == 1) {
+	    error(ERR_WARNING, "Unknown section attribute '%s' ignored on"
+                  " declaration of section `%s'", q, name);
+	}
     }
 
-    if (!strcmp(name, ".comment") ||
-        !strcmp(name, ".shstrtab") ||
-        !strcmp(name, ".symtab") || !strcmp(name, ".strtab")) {
+    if (!strcmp(name, ".shstrtab") ||
+        !strcmp(name, ".symtab") ||
+        !strcmp(name, ".strtab")) {
         error(ERR_NONFATAL, "attempt to redefine reserved section"
               "name `%s'", name);
         return NO_SEG;
@@ -527,25 +447,19 @@ static int32_t elf_section_names(char *name, int pass, int *bits)
         if (!strcmp(name, sects[i]->name))
             break;
     if (i == nsects) {
-        if (!strcmp(name, ".text"))
-            i = elf_make_section(name, SHT_PROGBITS,
-                                 SHF_ALLOC | SHF_EXECINSTR, 16);
-        else if (!strcmp(name, ".rodata"))
-            i = elf_make_section(name, SHT_PROGBITS, SHF_ALLOC, 4);
-        else if (!strcmp(name, ".data"))
-            i = elf_make_section(name, SHT_PROGBITS,
-                                 SHF_ALLOC | SHF_WRITE, 4);
-        else if (!strcmp(name, ".bss"))
-            i = elf_make_section(name, SHT_NOBITS,
-                                 SHF_ALLOC | SHF_WRITE, 4);
-        else
-            i = elf_make_section(name, SHT_PROGBITS, SHF_ALLOC, 1);
-        if (type)
-            sects[i]->type = type;
-        if (align)
-            sects[i]->align = align;
-        sects[i]->flags &= ~flags_and;
-        sects[i]->flags |= flags_or;
+	const struct elf_known_section *ks = elf_known_sections;
+
+	while (ks->name) {
+	    if (!strcmp(name, ks->name))
+		break;
+	    ks++;
+	}
+
+	type = type ? type : ks->type;
+	align = align ? align : ks->align;
+	flags = (ks->flags & ~flags_and) | flags_or;
+
+	i = elf_make_section(name, type, flags, align);
     } else if (pass == 1) {
           if ((type && sects[i]->type != type)
              || (align && sects[i]->align != align)
@@ -577,7 +491,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
          */
         if (strcmp(name, "..gotpc") && strcmp(name, "..gotoff") &&
             strcmp(name, "..got") && strcmp(name, "..plt") &&
-            strcmp(name, "..sym"))
+            strcmp(name, "..sym") && strcmp(name, "..gottpoff"))
             error(ERR_NONFATAL, "unrecognised special symbol `%s'", name);
         return;
     }
@@ -625,6 +539,8 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
 
     lastsym = sym = saa_wstruct(syms);
 
+    memset(&sym->symv, 0, sizeof(struct rbtree));
+
     sym->strpos = pos;
     sym->type = is_global ? SYM_GLOBAL : 0;
     sym->other = STV_DEFAULT;
@@ -651,7 +567,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
 
     if (is_global == 2) {
         sym->size = offset;
-        sym->value = 0;
+        sym->symv.key = 0;
         sym->section = SHN_COMMON;
         /*
          * We have a common variable. Check the special text to see
@@ -660,17 +576,18 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
          */
         if (special) {
             bool err;
-            sym->value = readnum(special, &err);
+            sym->symv.key = readnum(special, &err);
             if (err)
                 error(ERR_NONFATAL, "alignment constraint `%s' is not a"
                       " valid number", special);
-            else if ((sym->value | (sym->value - 1)) != 2 * sym->value - 1)
+            else if ((sym->symv.key | (sym->symv.key - 1))
+		     != 2 * sym->symv.key - 1)
                 error(ERR_NONFATAL, "alignment constraint `%s' is not a"
                       " power of two", special);
         }
         special_used = true;
     } else
-        sym->value = (sym->section == SHN_UNDEF ? 0 : offset);
+        sym->symv.key = (sym->section == SHN_UNDEF ? 0 : offset);
 
     if (sym->type == SYM_GLOBAL) {
         /*
@@ -687,16 +604,14 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
             bsym = raa_write(bsym, segment, nglobs);
         } else if (sym->section != SHN_ABS) {
             /*
-             * This is a global symbol; so we must add it to the linked
-             * list of global symbols in its section. We'll push it on
-             * the beginning of the list, because it doesn't matter
-             * much which end we put it on and it's easier like this.
+             * This is a global symbol; so we must add it to the rbtree
+             * of global symbols in its section.
              *
              * In addition, we check the special text for symbol
              * type and size information.
              */
-            sym->next = sects[sym->section - 1]->gsyms;
-            sects[sym->section - 1]->gsyms = sym;
+	    sects[sym->section-1]->gsyms =
+		rb_insert(sects[sym->section-1]->gsyms, &sym->symv);
 
             if (special) {
                 int n = strcspn(special, " \t");
@@ -762,6 +677,13 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
                 }
                 special_used = true;
             }
+            /*
+             * If TLS segment, mark symbol accordingly.
+             */
+            if (sects[sym->section - 1]->flags & SHF_TLS) {
+                sym->type &= 0xf0;
+                sym->type |= STT_TLS;
+            }
         }
         sym->globnum = nglobs;
         nglobs++;
@@ -821,12 +743,13 @@ static void elf_add_reloc(struct Section *sect, int32_t segment,
  * isn't even necessarily sorted.
  */
 static void elf_add_gsym_reloc(struct Section *sect,
-                               int32_t segment, int64_t offset, int64_t pcrel,
+                               int32_t segment, uint64_t offset, int64_t pcrel,
 			       int type, bool exact)
 {
     struct Reloc *r;
     struct Section *s;
-    struct Symbol *sym, *sm;
+    struct Symbol *sym;
+    struct rbtree *srb;
     int i;
 
     /*
@@ -851,34 +774,20 @@ static void elf_add_gsym_reloc(struct Section *sect,
 	return;
     }
 
-    if (exact) {
-        /*
-         * Find a symbol pointing _exactly_ at this one.
-         */
-        for (sym = s->gsyms; sym; sym = sym->next)
-            if (sym->value == offset)
-                break;
-	if (!sym) {
-	    error(ERR_NONFATAL, "unable to find a suitable global symbol"
-		  " for this reference");
-	    return;
-	}
-    } else {
-        /*
-         * Find the nearest symbol below this one.
-         */
-        sym = NULL;
-        for (sm = s->gsyms; sm; sm = sm->next)
-            if (sm->value <= offset && (!sym || sm->value > sym->value))
-                sym = sm;
+    srb = rb_search(s->gsyms, offset);
+    if (!srb || (exact && srb->key != offset)) {
+	error(ERR_NONFATAL, "unable to find a suitable global symbol"
+	      " for this reference");
+	return;
     }
+    sym = container_of(srb, struct Symbol, symv);
 
     r = *sect->tail = nasm_malloc(sizeof(struct Reloc));
     sect->tail = &r->next;
     r->next = NULL;
 
     r->address = sect->len;
-    r->offset = offset - pcrel - sym->value;
+    r->offset = offset - pcrel - sym->symv.key;
     r->symbol = GLOBAL_TEMP_BASE + sym->globnum;
     r->type = type;
 
@@ -943,20 +852,7 @@ static void elf_out(int32_t segto, const void *data,
     if (s->type == SHT_NOBITS && type != OUT_RESERVE) {
         error(ERR_WARNING, "attempt to initialize memory in"
               " BSS section `%s': ignored", s->name);
-	switch (type) {
-	case OUT_REL2ADR:
-	    size = 2;
-	    break;
-	case OUT_REL4ADR:
-	    size = 4;
-	    break;
-	case OUT_REL8ADR:
-	    size = 8;
-	    break;
-	default:
-	    break;		/* size is already set */
-	}
-        s->len += size;
+        s->len += realsize(type, size);
         return;
     }
 
@@ -1010,7 +906,7 @@ static void elf_out(int32_t segto, const void *data,
 	    } else if (wrt == elf_gotoff_sect + 1) {
 		if (size != 8) {
 		    error(ERR_NONFATAL, "ELF64 requires ..gotoff "
-			  "references to be qword absolute");
+			  "references to be qword");
 		} else {
 		    elf_add_reloc(s, segment, addr, R_X86_64_GOTOFF64);
 		    addr = 0;
@@ -1111,6 +1007,10 @@ static void elf_out(int32_t segto, const void *data,
 		       wrt == elf_got_sect + 1) {
 		error(ERR_NONFATAL, "ELF64 requires ..gotoff references to be "
 		      "qword absolute");
+            } else if (wrt == elf_gottpoff_sect + 1) {
+                elf_add_gsym_reloc(s, segment, addr+size, size,
+				   R_X86_64_GOTTPOFF, true);
+		addr = 0;
             } else {
                 error(ERR_NONFATAL, "ELF64 format does not support this"
                       " use of WRT");
@@ -1138,7 +1038,10 @@ static void elf_out(int32_t segto, const void *data,
             } else if (wrt == elf_gotoff_sect + 1 ||
 		       wrt == elf_got_sect + 1) {
 		error(ERR_NONFATAL, "ELF64 requires ..gotoff references to be "
-		      "qword absolute");
+		      "absolute");
+            } else if (wrt == elf_gottpoff_sect + 1) {
+		error(ERR_NONFATAL, "ELF64 requires ..gottpoff references to be "
+		      "dword");
             } else {
                 error(ERR_NONFATAL, "ELF64 format does not support this"
                       " use of WRT");
@@ -1151,10 +1054,7 @@ static void elf_out(int32_t segto, const void *data,
 static void elf_write(void)
 {
     int align;
-    int scount;
     char *p;
-    int commlen;
-    char comment[64];
     int i;
 
     struct SAA *symtab;
@@ -1162,18 +1062,16 @@ static void elf_write(void)
 
     /*
      * Work out how many sections we will have. We have SHN_UNDEF,
-     * then the flexible user sections, then the four fixed
-     * sections `.comment', `.shstrtab', `.symtab' and `.strtab',
-     * then optionally relocation sections for the user sections.
+     * then the flexible user sections, then the fixed sections
+     * `.shstrtab', `.symtab' and `.strtab', then optionally
+     * relocation sections for the user sections.
      */
+    nsections = sec_numspecial + 1;
     if (of_elf64.current_dfmt == &df_stabs)
-        nsections = 8;
+        nsections += 3;
     else if (of_elf64.current_dfmt == &df_dwarf)
-        nsections = 15;
-    else
-        nsections = 5;          /* SHN_UNDEF and the fixed ones */
+        nsections += 10;
 
-    add_sectname("", ".comment");
     add_sectname("", ".shstrtab");
     add_sectname("", ".symtab");
     add_sectname("", ".strtab");
@@ -1212,20 +1110,13 @@ static void elf_write(void)
         add_sectname("", ".debug_loc");
     }
 
-    /*
-     * Do the comment.
-     */
-    *comment = '\0';
-    commlen =
-        2 + sprintf(comment + 1, "The Netwide Assembler %s", NASM_VER);
-
     /*
      * Output the ELF header.
      */
     fwrite("\177ELF\2\1\1", 7, 1, elffp);
     fputc(elf_osabi, elffp);
     fputc(elf_abiver, elffp);
-    fwrite("\0\0\0\0\0\0\0", 7, 1, elffp);
+    fwritezero(7, elffp);
     fwriteint16_t(ET_REL, elffp);      /* relocatable file */
     fwriteint16_t(EM_X86_64, elffp);      /* processor ID */
     fwriteint32_t(1L, elffp);      /* EV_CURRENT file format version */
@@ -1239,8 +1130,8 @@ static void elf_write(void)
     fwriteint16_t(0, elffp);      /* still no program header table */
     fwriteint16_t(sizeof(Elf64_Shdr), elffp);   /* size of section header */
     fwriteint16_t(nsections, elffp);      /* number of sections */
-    fwriteint16_t(nsects + 2, elffp);     /* string table section index for
-                                         * section header table */
+    fwriteint16_t(sec_shstrtab, elffp);   /* string table section index for
+					   * section header table */
 
     /*
      * Build the symbol table and relocation tables.
@@ -1260,33 +1151,43 @@ static void elf_write(void)
     elf_foffs += align;
     elf_nsect = 0;
     elf_sects = nasm_malloc(sizeof(*elf_sects) * nsections);
-    elf_section_header(0, 0, 0, NULL, false, 0L, 0, 0, 0, 0);   /* SHN_UNDEF */
-    scount = 1;                 /* needed for the stabs debugging to track the symtable section */
+
+    /* SHN_UNDEF */
+    elf_section_header(0, SHT_NULL, 0, NULL, false, 0, SHN_UNDEF, 0, 0, 0);
     p = shstrtab + 1;
+
+    /* The normal sections */
     for (i = 0; i < nsects; i++) {
         elf_section_header(p - shstrtab, sects[i]->type, sects[i]->flags,
                            (sects[i]->type == SHT_PROGBITS ?
                             sects[i]->data : NULL), true,
                            sects[i]->len, 0, 0, sects[i]->align, 0);
         p += strlen(p) + 1;
-        scount++;               /* ditto */
     }
-    elf_section_header(p - shstrtab, 1, 0, comment, false, (int32_t)commlen, 0, 0, 1, 0);  /* .comment */
-    scount++;                   /* ditto */
+
+    /* .shstrtab */
+    elf_section_header(p - shstrtab, SHT_STRTAB, 0, shstrtab, false,
+		       shstrtablen, 0, 0, 1, 0);
     p += strlen(p) + 1;
-    elf_section_header(p - shstrtab, 3, 0, shstrtab, false, (int32_t)shstrtablen, 0, 0, 1, 0);     /* .shstrtab */
-    scount++;                   /* ditto */
+
+    /* .symtab */
+    elf_section_header(p - shstrtab, SHT_SYMTAB, 0, symtab, true,
+		       symtablen, sec_strtab, symtablocal, 4, 24);
     p += strlen(p) + 1;
-    elf_section_header(p - shstrtab, 2, 0, symtab, true, symtablen, nsects + 4, symtablocal, 4, 24);    /* .symtab */
-    symtabsection = scount;     /* now we got the symtab section index in the ELF file */
+
+    /* .strtab */
+    elf_section_header(p - shstrtab, SHT_STRTAB, 0, strs, true,
+		       strslen, 0, 0, 1, 0);
     p += strlen(p) + 1;
-    elf_section_header(p - shstrtab, 3, 0, strs, true, strslen, 0, 0, 1, 0);    /* .strtab */
+
+    /* The relocation sections */
     for (i = 0; i < nsects; i++)
         if (sects[i]->head) {
+            elf_section_header(p - shstrtab, SHT_RELA, 0, sects[i]->rel, true,
+                               sects[i]->rellen, sec_symtab, i + 1, 4, 24);
             p += strlen(p) + 1;
-            elf_section_header(p - shstrtab,SHT_RELA, 0, sects[i]->rel, true,
-                               sects[i]->rellen, nsects + 3, i + 1, 4, 24);
         }
+
     if (of_elf64.current_dfmt == &df_stabs) {
         /* for debugging information, create the last three sections
            which are the .stab , .stabstr and .rel.stab sections respectively */
@@ -1294,61 +1195,69 @@ static void elf_write(void)
         /* this function call creates the stab sections in memory */
         stabs64_generate();
 
-        if ((stabbuf) && (stabstrbuf) && (stabrelbuf)) {
+        if (stabbuf && stabstrbuf && stabrelbuf) {
+            elf_section_header(p - shstrtab, SHT_PROGBITS, 0, stabbuf, false,
+			       stablen, sec_stabstr, 0, 4, 12);
             p += strlen(p) + 1;
-            elf_section_header(p - shstrtab, 1, 0, stabbuf, false, stablen,
-                               nsections - 2, 0, 4, 12);
 
-            p += strlen(p) + 1;
-            elf_section_header(p - shstrtab, 3, 0, stabstrbuf, false,
+            elf_section_header(p - shstrtab, SHT_STRTAB, 0, stabstrbuf, false,
                                stabstrlen, 0, 0, 4, 0);
-
             p += strlen(p) + 1;
+
             /* link -> symtable  info -> section to refer to */
-            elf_section_header(p - shstrtab, 9, 0, stabrelbuf, false,
-                               stabrellen, symtabsection, nsections - 3, 4,
-                               16);
+            elf_section_header(p - shstrtab, SHT_REL, 0, stabrelbuf, false,
+                               stabrellen, symtabsection, sec_stab, 4, 16);
+            p += strlen(p) + 1;
         }
     }
     else if (of_elf64.current_dfmt == &df_dwarf) {
             /* for dwarf debugging information, create the ten dwarf sections */
 
             /* this function call creates the dwarf sections in memory */
-            if (dwarf_fsect) dwarf64_generate();
+            if (dwarf_fsect)
+		dwarf64_generate();
 
-            p += strlen(p) + 1;
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, arangesbuf, false,
                                arangeslen, 0, 0, 1, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_RELA, 0, arangesrelbuf, false,
                                arangesrellen, symtabsection, debug_aranges, 1, 24);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, pubnamesbuf, false,
                                pubnameslen, 0, 0, 1, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, infobuf, false,
                                infolen, 0, 0, 1, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_RELA, 0, inforelbuf, false,
                                inforellen, symtabsection, debug_info, 1, 24);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, abbrevbuf, false,
                                abbrevlen, 0, 0, 1, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, linebuf, false,
                                linelen, 0, 0, 1, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_RELA, 0, linerelbuf, false,
                                linerellen, symtabsection, debug_line, 1, 24);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, framebuf, false,
                                framelen, 0, 0, 8, 0);
             p += strlen(p) + 1;
+
             elf_section_header(p - shstrtab, SHT_PROGBITS, 0, locbuf, false,
                                loclen, 0, 0, 1, 0);
-
+            p += strlen(p) + 1;
     }
-    fwrite(align_str, align, 1, elffp);
+    fwritezero(align, elffp);
 
     /*
      * Now output the sections.
@@ -1417,7 +1326,7 @@ static struct SAA *elf_build_symtab(int32_t *len, int32_t *local)
         WRITECHAR(p, sym->type);        /* type and binding */
         WRITECHAR(p, sym->other);	/* visibility */
         WRITESHORT(p, sym->section);    /* index into section header table */
-        WRITEDLONG(p, (int64_t)sym->value); /* value of symbol */
+        WRITEDLONG(p, (int64_t)sym->symv.key); /* value of symbol */
         WRITEDLONG(p, (int64_t)sym->size);  /* size of symbol */
         saa_wbytes(s, entry, 24L);
         *len += 24;
@@ -1472,7 +1381,7 @@ static struct SAA *elf_build_symtab(int32_t *len, int32_t *local)
         WRITECHAR(p, sym->type);        /* type and binding */
         WRITECHAR(p, sym->other);       /* visibility */
         WRITESHORT(p, sym->section);
-        WRITEDLONG(p, (int64_t)sym->value);
+        WRITEDLONG(p, (int64_t)sym->symv.key);
         WRITEDLONG(p, (int64_t)sym->size);
         saa_wbytes(s, entry, 24L);
         *len += 24;
@@ -1485,6 +1394,7 @@ static struct SAA *elf_build_reltab(uint64_t *len, struct Reloc *r)
 {
     struct SAA *s;
     uint8_t *p, entry[24];
+    int32_t global_offset;
 
     if (!r)
         return NULL;
@@ -1492,18 +1402,23 @@ static struct SAA *elf_build_reltab(uint64_t *len, struct Reloc *r)
     s = saa_init(1L);
     *len = 0;
 
+    /*
+     * How to onvert from a global placeholder to a real symbol index;
+     * the +2 refers to the two special entries, the null entry and
+     * the filename entry.
+     */
+    global_offset = -GLOBAL_TEMP_BASE + nsects + nlocals + ndebugs + 2;
+
     while (r) {
-        int64_t sym = r->symbol;
+        int32_t sym = r->symbol;
 
         if (sym >= GLOBAL_TEMP_BASE)
-        {
-           if (of_elf64.current_dfmt == &df_dwarf)
-              sym += -GLOBAL_TEMP_BASE + (nsects + 5) + nlocals;
-           else   sym += -GLOBAL_TEMP_BASE + (nsects + 2) + nlocals;
-        }
+	    sym += global_offset;
+
         p = entry;
         WRITEDLONG(p, r->address);
-        WRITEDLONG(p, (sym << 32) + r->type);
+	WRITELONG(p, r->type);
+	WRITELONG(p, sym);
 	WRITEDLONG(p, r->offset);
         saa_wbytes(s, entry, 24L);
         *len += 24;
@@ -1549,7 +1464,7 @@ static void elf_write_sections(void)
                 saa_fpwrite(elf_sects[i].data, elffp);
             else
                 fwrite(elf_sects[i].data, len, 1, elffp);
-            fwrite(align_str, align, 1, elffp);
+            fwritezero(align, elffp);
         }
 }
 
@@ -1622,9 +1537,9 @@ static int elf_set_info(enum geninfo type, char **val)
     return 0;
 }
 static struct dfmt df_dwarf = {
-    "ELF64 (X86_64) dwarf debug format for Linux",
+    "ELF64 (x86-64) dwarf debug format for Linux/Unix",
     "dwarf",
-    debug64_init,
+    dwarf64_init,
     dwarf64_linenum,
     debug64_deflabel,
     debug64_directive,
@@ -1633,9 +1548,9 @@ static struct dfmt df_dwarf = {
     dwarf64_cleanup
 };
 static struct dfmt df_stabs = {
-    "ELF64 (X86_64) stabs debug format for Linux",
+    "ELF64 (x86-64) stabs debug format for Linux/Unix",
     "stabs",
-    debug64_init,
+    null_debug_init,
     stabs64_linenum,
     debug64_deflabel,
     debug64_directive,
@@ -1644,14 +1559,14 @@ static struct dfmt df_stabs = {
     stabs64_cleanup
 };
 
-struct dfmt *elf64_debugs_arr[3] = { &df_stabs, &df_dwarf, NULL };
+struct dfmt *elf64_debugs_arr[3] = { &df_dwarf, &df_stabs, NULL };
 
 struct ofmt of_elf64 = {
     "ELF64 (x86_64) object files (e.g. Linux)",
     "elf64",
-    NULL,
+    0,
     elf64_debugs_arr,
-    &null_debug_form,
+    &df_stabs,
     elf_stdmac,
     elf_init,
     elf_set_info,
@@ -1665,15 +1580,8 @@ struct ofmt of_elf64 = {
 };
 
 /* common debugging routines */
-void debug64_init(struct ofmt *of, void *id, FILE * fp, efunc error)
-{
-    (void)of;
-    (void)id;
-    (void)fp;
-    (void)error;
-}
-void debug64_deflabel(char *name, int32_t segment, int64_t offset, int is_global,
-                    char *special)
+static void debug64_deflabel(char *name, int32_t segment, int64_t offset,
+			     int is_global, char *special)
 {
     (void)name;
     (void)segment;
@@ -1682,13 +1590,13 @@ void debug64_deflabel(char *name, int32_t segment, int64_t offset, int is_global
     (void)special;
 }
 
-void debug64_directive(const char *directive, const char *params)
+static void debug64_directive(const char *directive, const char *params)
 {
     (void)directive;
     (void)params;
 }
 
-void debug64_typevalue(int32_t type)
+static void debug64_typevalue(int32_t type)
 {
     int32_t stype, ssize;
     switch (TYM_TYPE(type)) {
@@ -1724,6 +1632,10 @@ void debug64_typevalue(int32_t type)
             ssize = 16;
             stype = STT_OBJECT;
             break;
+        case TY_YWORD:
+	    ssize = 32;
+	    stype = STT_OBJECT;
+	    break;
         case TY_COMMON:
             ssize = 0;
             stype = STT_COMMON;
@@ -1753,8 +1665,7 @@ void debug64_typevalue(int32_t type)
 
 /* stabs debugging routines */
 
-
-void stabs64_linenum(const char *filename, int32_t linenumber, int32_t segto)
+static void stabs64_linenum(const char *filename, int32_t linenumber, int32_t segto)
 {
     (void)segto;
     if (!stabs_filename) {
@@ -1777,7 +1688,7 @@ void stabs64_linenum(const char *filename, int32_t linenumber, int32_t segto)
 }
 
 
-void stabs64_output(int type, void *param)
+static void stabs64_output(int type, void *param)
 {
     struct symlininfo *s;
     struct linelist *el;
@@ -1817,7 +1728,7 @@ void stabs64_output(int type, void *param)
 
 /* for creating the .stab , .stabstr and .rel.stab sections in memory */
 
-void stabs64_generate(void)
+static void stabs64_generate(void)
 {
     int i, numfiles, strsize, numstabs = 0, currfile, mainfileindex;
     uint8_t *sbuf, *ssbuf, *rbuf, *sptr, *rptr;
@@ -1952,7 +1863,7 @@ void stabs64_generate(void)
     stabstrbuf = ssbuf;
 }
 
-void stabs64_cleanup(void)
+static void stabs64_cleanup(void)
 {
     struct linelist *ptr, *del;
     if (!stabslines)
@@ -1971,9 +1882,18 @@ void stabs64_cleanup(void)
         nasm_free(stabstrbuf);
 }
 /* dwarf routines */
+static void dwarf64_init(struct ofmt *of, void *id, FILE * fp, efunc error)
+{
+    (void)of;
+    (void)id;
+    (void)fp;
+    (void)error;
 
+    ndebugs = 3;		/* 3 debug symbols */
+}
 
-void dwarf64_linenum(const char *filename, int32_t linenumber, int32_t segto)
+static void dwarf64_linenum(const char *filename, int32_t linenumber,
+			    int32_t segto)
 {
     (void)segto;
     dwarf64_findfile(filename);
@@ -1982,7 +1902,7 @@ void dwarf64_linenum(const char *filename, int32_t linenumber, int32_t segto)
 }
 
 /* called from elf_out with type == TY_DEBUGSYMLIN */
-void dwarf64_output(int type, void *param)
+static void dwarf64_output(int type, void *param)
 {
   int ln, aa, inx, maxln, soc;
   struct symlininfo *s;
@@ -2045,9 +1965,8 @@ void dwarf64_output(int type, void *param)
 }
 
 
-void dwarf64_generate(void)
+static void dwarf64_generate(void)
 {
-    static const char nasm_signature[] = "NASM " NASM_VER;
     uint8_t *pbuf;
     int indx;
     struct linelist *ftentry;
@@ -2282,7 +2201,7 @@ void dwarf64_generate(void)
     WRITEDLONG(pbuf,0);		/* null  ending offset */
 }
 
-void dwarf64_cleanup(void)
+static void dwarf64_cleanup(void)
 {
     if (arangesbuf)
         nasm_free(arangesbuf);
@@ -2305,7 +2224,7 @@ void dwarf64_cleanup(void)
     if (locbuf)
         nasm_free(locbuf);
 }
-void dwarf64_findfile(const char * fname)
+static void dwarf64_findfile(const char * fname)
 {
    int finx;
    struct linelist *match;
@@ -2350,7 +2269,7 @@ void dwarf64_findfile(const char * fname)
    }
 }
 /*  */
-void dwarf64_findsect(const int index)
+static void dwarf64_findsect(const int index)
 {
    int sinx;
    struct sectlist *match;
