@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 ## --------------------------------------------------------------------------
 ##   
-##   Copyright 1996-2016 The NASM Authors - All Rights Reserved
+##   Copyright 1996-2009 The NASM Authors - All Rights Reserved
 ##   See the file AUTHORS included with the NASM distribution for
 ##   the specific copyright holders.
 ##
@@ -56,13 +56,13 @@
 # NASM_VER		-- whole version number as a string
 #
 # version.mac:
-# __?NASM_MAJOR?__
-# __?NASM_MINOR?__
-# __?NASM_SUBMINOR?__
-# __?NASM_PATCHLEVEL?__
-# __?NASM_SNAPSHOT?__
-# __?NASM_VERSION_ID?__
-# __?NASM_VER?__
+# __NASM_MAJOR__
+# __NASM_MINOR__
+# __NASM_SUBMINOR__
+# __NASM_PATCHLEVEL__
+# __NASM_SNAPSHOT__
+# __NASM_VERSION_ID__
+# __NASM_VER__
 #
 
 ($what) = @ARGV;
@@ -70,11 +70,7 @@
 $line = <STDIN>;
 chomp $line;
 
-undef $maj;
-undef $min;
-undef $smin;
-undef $plvl;
-undef $tail;
+undef $man, $min, $smin, $plvl, $tail;
 $is_rc = 0;
 
 if ( $line =~ /^([0-9]+)\.([0-9]+)(.*)$/ ) {
@@ -147,16 +143,15 @@ if ( $what eq 'h' ) {
     printf "#define NASM_VER            \"%s\"\n", $line;
     print  "#endif /* NASM_VERSION_H */\n";
 } elsif ( $what eq 'mac' ) {
-    print  "STD: version\n";
-    printf "%%define __?NASM_MAJOR?__ %d\n", $nmaj;
-    printf "%%define __?NASM_MINOR?__ %d\n", $nmin;
-    printf "%%define __?NASM_SUBMINOR?__ %d\n", $nsmin;
-    printf "%%define __?NASM_PATCHLEVEL?__ %d\n", $nplvl;
+    printf "%%define __NASM_MAJOR__ %d\n", $nmaj;
+    printf "%%define __NASM_MINOR__ %d\n", $nmin;
+    printf "%%define __NASM_SUBMINOR__ %d\n", $nsmin;
+    printf "%%define __NASM_PATCHLEVEL__ %d\n", $nplvl;
     if (defined($snapshot)) {
-	printf "%%define __?NASM_SNAPSHOT?__ %d\n", $snapshot;
+	printf "%%define __NASM_SNAPSHOT__ %d\n", $snapshot;
     }
-    printf "%%define __?NASM_VERSION_ID?__ 0%08Xh\n", $nasm_id;
-    printf "%%define __?NASM_VER?__ \"%s\"\n", $line;
+    printf "%%define __NASM_VERSION_ID__ 0%08Xh\n", $nasm_id;
+    printf "%%define __NASM_VER__ \"%s\"\n", $line;
 } elsif ( $what eq 'sed' ) {
     printf "s/\@\@NASM_MAJOR\@\@/%d/g\n", $nmaj;
     printf "s/\@\@NASM_MINOR\@\@/%d/g\n", $nmin;
@@ -183,9 +178,6 @@ if ( $what eq 'h' ) {
     print $nasm_id, "\n";	 # Print ID in decimal
 } elsif ( $what eq 'xid' ) {
     printf "0x%08x\n", $nasm_id; # Print ID in hexadecimal
-} elsif ( $what eq 'docsrc' ) {
-    printf "\\M{version}{%s}\n", $line;
-    printf "\\M{subtitle}{version %s}\n", $line;
 } else {
     die "$0: Unknown output: $what\n";
 }
